
unit Forme;
{$MODE OBJFPC}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, BGRABitmap, BGRABitmapTypes, BGRAGradientScanner, Grille, IniFiles,
  LCLType;

type
  TAnimData = record
    fall: boolean;
    x, y: integer;
  end;
  
  TForm1 = class(TForm)
    Button1: TButton;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    data: TAnimData;
    buttons: array[1..7]of TButton;
    pixels: array[1..7, 1..7]of TBGRAPixel;
    grille: TGrille;
    image: TBGRABitmap;
    pion: TPion;
    procedure AjouterPion(x: integer);
    procedure DrawDisk(x, y: integer; c: TBGRAPixel);
    procedure PaintImage;
    procedure Restart;
  end;

var
  Form1: TForm1;

implementation
{$R *.lfm}

uses
  Couleurs, Moteur;

type
  TJoueur = record
    auto: boolean;
    nom: string;
  end;

var
  colors  : array[1..4]of TBGRAPixel;
  chaines : array[1..4]of string;
  joueurs : array[1..2]of TJoueur;
  fin     : boolean = false;
  histoire: string = '';

procedure SetPixel(var aPixel: TBGRAPixel; const c: longword);
begin
  aPixel.blue  := (c and $FF0000) shr 16;
  aPixel.green := (c and $00FF00) shr 8;
  aPixel.red   := (c and $0000FF);
  aPixel.alpha := $FF;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  x: integer;
begin
  for x := 1 to 7 do
  begin
    buttons[x] := TButton.Create(Self);
    with buttons[x] do
    begin
      Parent := self;
      Name   := Chr(x - 1 + Ord('A'));
      Left   := 48 * (x - 1) + 24 + 4;
      Top    := 384 + 12;
      Width  := 40;
      Height := 24;
      Tag    := x;
      onClick := @Button1Click;
    end;
  end;

  data.fall := false;
  image := TBGRABitmap.Create(8 * 48, 8 * 48);

  Vider(grille);

  pion := blanc;
  fin := false;
  histoire := '';
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  x: integer;
begin
  if data.fall then
    exit;

  if fin then
  begin
    Restart;
    exit;
  end;

  x := (Sender as TButton).Tag;
  if grille[x, 6] = rien then
    AjouterPion(x);
end;

procedure TForm1.FormActivate(Sender: TObject);
var
  ini: TIniFile;
  lang: string;
  x, y: integer;
  pal: string;
begin
  ini := TIniFile.Create(ExtractFilePath(ParamStr(0)) + 'options.ini');

  joueurs[1].nom := ini.ReadString('joueurs', 'blanc', '');
  joueurs[2].nom := ini.ReadString('joueurs', 'noir', '');

  if (joueurs[1].nom = '') or (joueurs[2].nom = '') then
  begin
    if joueurs[1].nom = '' then
    begin
      joueurs[1].nom := 'utilisateur';
      ini.WriteString('joueurs', 'blanc', joueurs[1].nom);
    end;
    if joueurs[2].nom = '' then
    begin
      joueurs[2].nom := 'moteur';
      ini.WriteString('joueurs', 'noir', joueurs[2].nom);
    end;
    ini.UpdateFile;
  end;

  joueurs[1].auto := (joueurs[1].nom = 'moteur');
  joueurs[2].auto := (joueurs[2].nom = 'moteur');

  lang := ini.ReadString('interface', 'language', '');

  if not ((lang = 'en') or (lang = 'de') or (lang = 'fr')) then
  begin
    lang := 'fr';
    ini.WriteString('interface', 'language', lang);
    ini.UpdateFile;
  end;

  if lang = 'en' then
  begin
    chaines[1] := 'Four-in-a-row';
    chaines[2] := 'White wins.';
    chaines[3] := 'Black wins.';
    chaines[4] := 'The grid is full.';
  end;

  if lang = 'de' then
  begin
    chaines[1] := 'Vier-in-einer-Reihe';
    chaines[2] := 'Weiß gewinnt.';
    chaines[3] := 'Schwarz gewinnt.';
    chaines[4] := 'Das Spielfeld ist voll.';
  end;

  if lang = 'fr' then
  begin
    chaines[1] := 'Quatre-en-ligne';
    chaines[2] := 'Le Blanc gagne.';
    chaines[3] := 'Le Noir gagne.';
    chaines[4] := 'La grille est pleine.';
  end;

  Form1.Caption := chaines[1];

  pal := ini.ReadString('interface', 'palette', '');

  if not ((pal = 'green') or (pal = 'indigo') or (pal = 'blue') or (pal = 'gray')) then
  begin
    pal := 'gray';
    ini.WriteString('interface', 'palette', pal);
    ini.UpdateFile;
  end;

  if pal = 'green' then
  begin
    SetPixel(colors[1], ForestGreen);
    SetPixel(colors[2], LimeGreen);
    SetPixel(colors[3], Lemon);
    SetPixel(colors[4], DarkOrange);
  end;

  if pal = 'indigo' then
  begin
    SetPixel(colors[1], DarkIndigo);
    SetPixel(colors[2], Indigo);
    SetPixel(colors[3], Mauve);
    SetPixel(colors[4], DarkOrchid);
  end;

  if pal = 'blue' then
  begin
    SetPixel(colors[1], DarkSlateBlue);
    SetPixel(colors[2], SlateBlue);
    SetPixel(colors[3], GoldenRod);
    SetPixel(colors[4], Cerise);
  end;

  if pal = 'gray' then
  begin
    SetPixel(colors[1], $707070);
    SetPixel(colors[2], $808080);
    SetPixel(colors[3], $C0C0C0);
    SetPixel(colors[4], $202020);
  end;

  for x := 1 to 7 do
    for y := 1 to 6 do
      pixels[x, y] := colors[2];

  for x := 1 to 7 do
    pixels[x, 7] := colors[1];

  ini.Free;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  image.Free;
end;

procedure TForm1.FormPaint(Sender: TObject);
var
  x, y: integer;
begin
  image.FillRect(0, 0, image.Width, image.Height, colors[1], dmSet);
  for x := 1 to 7 do
    for y := 1 to 7 do
      DrawDisk(
        48 * x - 24,
        7 * 48 - (48 * y - 24),
        pixels[x, y]
      );
  PaintImage;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  px: TBGRAPixel;
  msg: string;
  x: integer;
begin
  if fin then
    exit;

  if data.fall then
  begin
    if (data.y = 1) or (pixels[data.x, data.y - 1] <> colors[2]) then
    begin
      data.fall := false;

      msg := '';
      if PointsGrille(noir, grille) >= 4 then
        msg := chaines[3];
      if (msg = '') and (PointsGrille(blanc, grille) >= 4) then
        msg := chaines[2];
      if (msg = '') and EstPleine(grille) then
        msg := chaines[4];

      if Length(msg) > 0 then
      begin
        fin := true;
        Application.MessageBox(
          pchar(msg),
          pchar(chaines[1]),
          MB_ICONINFORMATION
        );
      end;
    end else
    begin
      px := pixels[data.x, data.y];
      if data.y < 7 then
        pixels[data.x, data.y] := colors[2]
      else
        pixels[data.x, data.y] := colors[1];
      Dec(data.y);
      pixels[data.x, data.y] := px;
      FormPaint(nil);
    end;
  end else
  begin
    if ((pion = blanc) and joueurs[1].auto)
    or ((pion = noir ) and joueurs[2].auto) then
    begin
      x := Coup(histoire);
      AjouterPion(x);
    end;
  end;
end;

procedure TForm1.DrawDisk(x, y: integer; c: TBGRAPixel);
var
  img: TBGRABitmap;
  grad: TBGRACustomScanner;
begin
  if c = colors[1] then exit;
  img := TBGRABitmap.Create(48, 48);
  if c = colors[2] then
    grad := TBGRAConstantScanner.Create(c)
  else
    grad := TBGRAGradientScanner.Create(
      ApplyIntensityFast(c, 50000),
      ApplyIntensityFast(c, 16000),
      gtRadial,
      PointF(8, 8),
      PointF(48, 48)
    );
  img.FillEllipseAntialias(
    img.Width / 2,
    img.Height / 2,
    7 * img.Width / 16,
    7 * img.Height / 16,
    grad
  );
  grad.Free;
  image.PutImage(x, y, img, dmDrawWithTransparency);
  img.Free;
end;

procedure TForm1.PaintImage;
begin
  image.Draw(Canvas, 0, 0, True);
end;

procedure TForm1.Restart;
var
  x, y: integer;
begin
  for x := 1 to 7 do
    for y := 1 to 6 do
      pixels[x, y] := colors[2];

  for x := 1 to 7 do
    pixels[x, 7] := colors[1];

  Vider(grille);

  pion := blanc;
  fin := false;
  histoire := '';

  FormPaint(nil);
end;

procedure TForm1.AjouterPion(x: integer);
begin
  if pion = blanc then
    pixels[x, 7] := colors[3]
  else
    pixels[x, 7] := colors[4];

  FormPaint(nil);

  data.fall := true;
  data.x := x;
  data.y := 7;

  if Ajouter(grille, pion, x) then
  begin
    pion := noir * pion;
    histoire := histoire + IntToStr(x);
  end;
end;

end.
