
unit Grille;
{$MODE OBJFPC}

interface

const
  rien   =  0;
  pion   =  1;
  bord   =  2;
  noir   = -1;
  blanc  =  1;
  un     =  1;
  quatre =  4;
  six    =  6;
  sept   =  7;

type
  tPion   = noir..bord;
  tGrille = array[un - 1..sept + 1, un - 1..six + 1]of tPion;
  tLigne  = record
              x1, y1, x2, y2, dx, dy: integer;
            end;

procedure Vider(var g: tGrille);
procedure Copier(var g: tGrille; const h: tGrille);
function Ajouter(var g: tGrille; const p: tPion; const x: integer): boolean;
function EstVide(const g: tGrille): boolean;
function EstPleine(const g: tGrille): boolean;
function Chaine(const g: tGrille): string;
function PointsLigne(const p: tPion; const l: tLigne; const g: tGrille): integer;
function PointsGrille(const p: tPion; const g: tGrille): integer;

implementation

const
  cLigne: array[0..24]of tLigne = (
    (x1: 1; y1: 1; x2: 7; y2: 1; dx: 1; dy: 0),
    (x1: 1; y1: 2; x2: 7; y2: 2; dx: 1; dy: 0),
    (x1: 1; y1: 3; x2: 7; y2: 3; dx: 1; dy: 0),
    (x1: 1; y1: 4; x2: 7; y2: 4; dx: 1; dy: 0),
    (x1: 1; y1: 5; x2: 7; y2: 5; dx: 1; dy: 0),
    (x1: 1; y1: 6; x2: 7; y2: 6; dx: 1; dy: 0),
    (x1: 1; y1: 1; x2: 1; y2: 6; dx: 0; dy: 1),
    (x1: 2; y1: 1; x2: 2; y2: 6; dx: 0; dy: 1),
    (x1: 3; y1: 1; x2: 3; y2: 6; dx: 0; dy: 1),
    (x1: 4; y1: 1; x2: 4; y2: 6; dx: 0; dy: 1),
    (x1: 5; y1: 1; x2: 5; y2: 6; dx: 0; dy: 1),
    (x1: 6; y1: 1; x2: 6; y2: 6; dx: 0; dy: 1),
    (x1: 7; y1: 1; x2: 7; y2: 6; dx: 0; dy: 1),
    (x1: 1; y1: 1; x2: 7; y2: 6; dx: 1; dy: 1),
    (x1: 1; y1: 2; x2: 6; y2: 6; dx: 1; dy: 1),
    (x1: 1; y1: 3; x2: 5; y2: 6; dx: 1; dy: 1),
    (x1: 1; y1: 4; x2: 5; y2: 1; dx: 1; dy:-1),
    (x1: 1; y1: 5; x2: 6; y2: 1; dx: 1; dy:-1),
    (x1: 1; y1: 6; x2: 7; y2: 1; dx: 1; dy:-1),
    (x1: 7; y1: 1; x2: 7; y2: 6; dx:-1; dy: 1),
    (x1: 7; y1: 2; x2: 6; y2: 6; dx:-1; dy: 1),
    (x1: 7; y1: 3; x2: 5; y2: 6; dx:-1; dy: 1),
    (x1: 7; y1: 4; x2: 5; y2: 1; dx:-1; dy:-1),
    (x1: 7; y1: 5; x2: 6; y2: 1; dx:-1; dy:-1),
    (x1: 7; y1: 6; x2: 7; y2: 1; dx:-1; dy:-1)
  );

procedure Vider(var g: tGrille);
var
  x, y: integer;
begin
  for x := un - 1 to sept + 1 do
    for y := un - 1 to six + 1 do
      g[x, y] := bord;

  for x := un to sept do
    for y := un to six do
      g[x, y] := rien;
end;

procedure Copier(var g: tGrille; const h: tGrille);
var
  x, y: integer;
begin
  for x := un to sept do
    for y := un to six do
      g[x, y] := h[x, y];
end;

function Ajouter(var g: tGrille; const p: tPion; const x: integer): boolean;
var
  y: integer;
begin
  y := six;
  result := (x >= un) and (x <= sept) and (g[x, y] = rien);
  if result then
  begin
    while (y > un) and (g[x, y - 1] = rien) do
      Dec(y);
    g[x, y] := p;
  end;
end;

function EstVide(const g: tGrille): boolean;
var
  x: integer;
begin
  result := true;
  for x := un to sept do
    if g[x, un] <> rien then
    begin
      result := false;
      break;
    end;
end;

function EstPleine(const g: tGrille): boolean;
var
  x: integer;
begin
  result := true;
  for x := un to sept do
    if g[x, six] = rien then
    begin
      result := false;
      break;
    end;
end;

function Chaine(const g: tGrille): string;
const
  symbole: array[tPion]of char = ('O', '.', 'X', '#');
  canevas =
  '#################'#13#10 +
  '# x x x x x x x #'#13#10 +
  '# x x x x x x x #'#13#10 +
  '# x x x x x x x #'#13#10 +
  '# x x x x x x x #'#13#10 +
  '# x x x x x x x #'#13#10 +
  '# x x x x x x x #'#13#10 +
  '#################';
var
  x, y, i: integer;
begin
  result := canevas;
  for y := six downto un do
    for x := un to sept do
    begin
      i := 1 + 19 * (six - y + 1) + 2 * x;
      result[i] := symbole[g[x, y]];
    end;
end;

function PointsLigne(const p: tPion; const l: tLigne; const g: tGrille): integer;
var
  x, y, n, o, a, b: integer;
begin
  x := l.x1;
  y := l.y1;
  n := 0;
  o := 0;
  a := 0;
  b := 0;
  repeat
    if g[x,y] = p then
      Inc(n)
    else
      n := 0;
    if (g[x, y] = p) or (g[x, y] = rien) then
      Inc(o)
    else
      o := 0;
    if n > a then
      a := n;
    if o > b then
      b := o;
    Inc(x, l.dx);
    Inc(y, l.dy);
  until g[x, y] = bord;
  if b >= quatre then
    result := a
  else
    result := 0;
end;

function PointsGrille(const p: tPion; const g: tGrille): integer;
var
  i, j: integer;
begin
  result := 0;
  for i := 0 to 24 do
  begin
    j := PointsLigne(p, cLigne[i], g);
    if j > result then result := j;
  end;
end;

end.
