
unit grille;

{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

interface

const
  UN     = 1;
  QUATRE = 4;
  SIX    = 6;
  SEPT   = 7;
  
  VIDE   = '.';
  BLANC  = 'X';
  NOIR   = 'O';

function AutreCouleur(const aPion: char): char;

type
  tGrille = array[UN..SEPT, UN..SIX]of char;
  
const
  GRILLE_VIDE: tGrille = (
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE),
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE),
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE),
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE),
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE),
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE),
    (VIDE, VIDE, VIDE, VIDE, VIDE, VIDE)
  );
  
type
  tClasseGrille = class
    grille: tGrille;
    procedure Initialise(const aGrille: tGrille); overload;
    procedure Initialise(const aCoups: string); overload;
    function Texte: string;
    function Insere(const aPion: char; const aColonne: integer): boolean;
    procedure Retire(const aColonne: integer);
    function Points(const aPion: char): integer;
    function MeilleurCoup(const aPion: char): integer;
  end;

implementation

uses
  sysutils, fonctions;

function AutreCouleur(const aPion: char): char;
begin
  if aPion = BLANC then
    result := NOIR
  else
    result := BLANC;
end;

procedure tClasseGrille.Initialise(const aGrille: tGrille);
var
  x, y: integer;
begin
  for x := UN to SEPT do
    for y := UN to SIX do
      grille[x, y] := aGrille[x, y];
end;

procedure tClasseGrille.Initialise(const aCoups: string);
var
  i: integer;
  pion: char;
begin
  Initialise(GRILLE_VIDE);
  pion := BLANC;
  for i := 1 to Length(aCoups) do
  begin
    Insere(pion, Ord(aCoups[i]) - Ord('0'));
    Pion := AutreCouleur(pion);
  end;
end;

function tClasseGrille.Texte: string;
var
  x, y: integer;
begin
  result :=
  '   a b c d e f g'#13#10 +
  '6  X X X X X X X'#13#10 +
  '5  X X X X X X X'#13#10 +
  '4  X X X X X X X'#13#10 +
  '3  X X X X X X X'#13#10 +
  '2  X X X X X X X'#13#10 +
  '1  X X X X X X X';
  
  for x := UN to SEPT do
    for y := UN to SIX do
      result[18 * (SIX - y + 1) + 2 * x + 2] := grille[x, y];
end;

function tClasseGrille.Insere(const aPion: char; const aColonne: integer): boolean;
var
  y: integer;
begin
  y := SIX;
  if grille[aColonne, y] = VIDE then
  begin
    result := TRUE;
    while (y > UN) and (grille[aColonne, y - 1] = VIDE) do
      Dec(y);
    grille[aColonne, y] := aPion;
  end else
    result := FALSE;
end;

procedure tClasseGrille.Retire(const aColonne: integer);
var
  y: integer;
begin
  y := SIX;
  while (y >= UN) and (grille[aColonne, y] = VIDE) do
    Dec(y);
  if y >= UN then
    grille[aColonne, y] := VIDE;
end;

function tClasseGrille.Points(const aPion: char): integer;
const
  motifs: array[0..13, 0..1]of pChar = (
    ('OOOO', 'XXXX'),
    ('OOO.', 'XXX.'),
    ('.OOO', '.XXX'),
    ('OO.O', 'XX.X'),
    ('O.OO', 'X.XX'),
    ('OO..', 'XX..'),
    ('O.O.', 'X.X.'),
    ('O..O', 'X..X'),
    ('.O.O', '.X.X'),
    ('..OO', '..XX'),
    ('O...', 'X...'),
    ('.O..', '.X..'),
    ('..O.', '..X.'),
    ('...O', '...X')
  );
type
  tDemiDroite = record
    xo, yo, dx, dy: integer;
  end;
const
  demiDroites: array[0..24]of tDemiDroite = (
    (xo: 1; yo: 1; dx:+1; dy: 0),
    (xo: 1; yo: 2; dx:+1; dy: 0),
    (xo: 1; yo: 3; dx:+1; dy: 0),
    (xo: 1; yo: 4; dx:+1; dy: 0),
    (xo: 1; yo: 5; dx:+1; dy: 0),
    (xo: 1; yo: 6; dx:+1; dy: 0),
    (xo: 1; yo: 1; dx: 0; dy:+1),
    (xo: 2; yo: 1; dx: 0; dy:+1),
    (xo: 3; yo: 1; dx: 0; dy:+1),
    (xo: 4; yo: 1; dx: 0; dy:+1),
    (xo: 5; yo: 1; dx: 0; dy:+1),
    (xo: 6; yo: 1; dx: 0; dy:+1),
    (xo: 7; yo: 1; dx: 0; dy:+1),
    (xo: 1; yo: 1; dx:+1; dy:+1),
    (xo: 1; yo: 2; dx:+1; dy:+1),
    (xo: 1; yo: 3; dx:+1; dy:+1),
    (xo: 1; yo: 4; dx:+1; dy:-1),
    (xo: 1; yo: 5; dx:+1; dy:-1),
    (xo: 1; yo: 6; dx:+1; dy:-1),
    (xo: 7; yo: 1; dx:-1; dy:+1),
    (xo: 7; yo: 2; dx:-1; dy:+1),
    (xo: 7; yo: 3; dx:-1; dy:+1),
    (xo: 7; yo: 4; dx:-1; dy:-1),
    (xo: 7; yo: 5; dx:-1; dy:-1),
    (xo: 7; yo: 6; dx:-1; dy:-1)
  );
var
  i, j, k, l: integer;
  x, y: integer;
  segment: array[0..7]of char;
begin
  result := 0;
  l := Ord(aPion = BLANC);
  
  for i := Low(demiDroites) to High(demiDroites) do
  begin
    x := demiDroites[i].xo;
    y := demiDroites[i].yo;
    
    j := 0;
    FillChar(segment, SizeOf(segment), #0);
    repeat
      segment[j] := grille[x, y];
      Inc(x, demiDroites[i].dx);
      Inc(y, demiDroites[i].dy);
      Inc(j);
    until (x < UN) or (x > SEPT) or (y < UN) or (y > SIX);
    
    for j := Low(motifs) to High(motifs) do
      if StrPos(segment, motifs[j, l]) <> nil then
      begin
        case j of
          0:
            k := 75000;
          1..4:
            k := 2500;
          5..9:
            k := 50;
          else
            k := 1;
        end;
        Inc(result, k);
      end;
  end;
end;

function tClasseGrille.MeilleurCoup(const aPion: char): integer;
var
  i, j, k: integer;
  a, b, c: array[UN..SEPT]of integer;
begin
  for i := UN to SEPT do
  begin
    a[i] := Low(integer);
    if Insere(aPion, i) then
    begin
      a[i] := Points(aPion);
      Retire(i);
    end;
  end;
  
  if Max(a) >= 75000 then
    result := IndexOf(a, Max(a)) + UN
  else
  begin
    for i := UN to SEPT do
    begin
      a[i] := Low(integer);
      if Insere(aPion, i) then
      begin
        for j := UN to SEPT do
        begin
          b[j] := Low(integer);
          if Insere(AutreCouleur(aPion), j) then
          begin
            b[j] := Points(AutreCouleur(aPion));
            Retire(j);
          end;
        end;
        Retire(i);
        a[i] := Max(b);
      end;
    end;
    
    if Max(a) >= 2500 then
      result := IndexOf(a, Min(a, Low(integer))) + UN
    else
    begin
      for i := UN to SEPT do
      begin
        a[i] := Low(integer);
        if Insere(aPion, i) then
        begin
          for j := UN to SEPT do
          begin
            b[j] := Low(integer);
            if Insere(AutreCouleur(aPion), j) then
            begin
              for k := UN to SEPT do
              begin
                c[k] := Low(integer);
                if Insere(aPion, k) then
                begin
                  c[k] := Points(aPion);
                  Retire(k);
                end;
              end;
              b[j] := Max(c);
              Retire(j);
            end;
          end;
          a[i] := Min(b, Low(integer));
          Retire(i);
        end;
      end;
      
      result := IndexOf(a, Max(a)) + UN;
    end;
  end;
end;

end.
